package com.zyh.auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.zyh.auth.feign.AuthFeignService;
import com.zyh.auth.service.AuthService;
import com.zyh.common.constants.JwtConstants;
import com.zyh.common.constants.TokenConstants;
import com.zyh.common.domain.User;
import com.zyh.common.domain.request.UserRequest;
import com.zyh.common.domain.response.UserResponse;
import com.zyh.common.result.Result;
import com.zyh.common.utils.JwtUtils;
import com.zyh.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;

@Service
public class AuthServicempl implements AuthService {
    @Autowired
    RedisTemplate<String,String> redisTemplate;
    @Autowired
    HttpServletRequest request;
    @Autowired
    AuthFeignService authFeignService;
    @Override
    public Result<UserResponse> login(UserRequest userRequest) {
        if (StringUtils.isAnyBlank(userRequest.getPhone(), userRequest.getPwd())) {
            return Result.error("手机号或密码不能为空");
        }
        if (!rule(userRequest.getPhone())) {
            return Result.error("手机号格式错误");
        }
        Result<User> result = authFeignService.findPhone(userRequest.getPhone());
        User user = result.getData();
        if (user == null) {
            return Result.error("手机号未注册");
        }
        if (!userRequest.getPwd().equals(user.getPwd())) {
            return Result.error("密码错误");
        }
        String userkey = UUID.randomUUID().toString().replaceAll("-", "");
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY, userkey);
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY + userkey, JSONObject.toJSONString(user), 60, TimeUnit.MINUTES);
        UserResponse userResponse = new UserResponse();
        userResponse.setToken(token);
        userResponse.setExpirTime("60MIN");
        return Result.success(userResponse);
    }

    @Override
    public User userInfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(s, User.class);
        return user;
    }

    private boolean rule(String userPhone) {
        Pattern compile = Pattern.compile("^(?:(?:\\+|00)86)?1(?:(?:3[\\d])|(?:4[5-79])|(?:5[0-35-9])|(?:6[5-7])|(?:7[0-8])|(?:8[\\d])|(?:9[189]))\\d{8}$");
        return compile.matcher(userPhone).matches();
    }
}
