 package com.zyh.gateway.filters;

import com.zyh.common.constants.JwtConstants;
import com.zyh.common.constants.TokenConstants;
import com.zyh.common.utils.JwtUtils;
import com.zyh.common.utils.StringUtils;
import com.zyh.gateway.config.IgnoreWhiteConfig;
import com.zyh.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;

@Configuration
public class AuthFilters implements GatewayFilter, Ordered {

    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        //白名单放行
        List<String> whites = ignoreWhiteConfig.getWhites();
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        if (StringUtils.matches(path,whites)){
            return chain.filter(exchange);
        }
        //判断token是否为空
        HttpHeaders headers = request.getHeaders();
        String token = headers.getFirst(TokenConstants.TOKEN);
        if (StringUtils.isEmpty(token)){
            return GatewayUtils.errorResponse(exchange,"token为空111");
        }

        //token合法性
        try {
            JwtUtils.parseToken(token);
        } catch (Exception e) {
            return GatewayUtils.errorResponse(exchange,"token不合法");
        }
        //token时效性
        String userKey = JwtUtils.getUserKey(token);
        Boolean hasKey = stringRedisTemplate.hasKey(JwtConstants.USER_KEY + userKey);
        if (!hasKey){
            return GatewayUtils.errorResponse(exchange,"token失效");
        }
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
