package com.zyh.system.service.impl;

import com.zyh.common.domain.Area;
import com.zyh.common.domain.House;
import com.zyh.common.domain.vo.AreaVo;
import com.zyh.common.result.Result;
import com.zyh.common.utils.StringUtils;
import com.zyh.system.mapper.HouseMapper;
import com.zyh.system.service.HouseService;
import com.zyh.system.util.FastUtil;
import com.zyh.system.util.OssUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

@Service
public class HouseServicempl implements HouseService {
    @Autowired
    FastUtil fastUtil;
    @Autowired
    private HouseMapper houseMapper;

    @Override
    public Result addHouse(House house) {
        //校验参数

        checkHouse(house);

            Integer add = houseMapper.addHouse(house);
            if (add > 0) {
                return Result.success("添加成功");
            } else {
                return Result.error("添加失败");
            }
        }

    private void checkHouse(House house) {

        if (StringUtils.isEmpty(house.getName())) {
            throw new RuntimeException("名称不能为空");
        }
        if (StringUtils.isEmpty(house.getPlotId())) {
            throw new RuntimeException("所属小区编号不能为空");
        }
        if (house.getJurisdiction() == null) {
            throw new RuntimeException("所属辖区不能为空");
        }
    }

    @Override
    public Result<List<Area>> getJurisdictionTree(Integer parentId) {
        List<Area> list=houseMapper.getJurisdictionTree(parentId);

        return Result.success(list);
    }

    @Override
    public Result<List<House>> getHouse() {
        List<House> list=houseMapper.getHouse();
        return Result.success(list);
    }

    @Override
    public Result sendPic(MultipartFile pic) {


        String sendPic = "";
        sendPic= OssUtil.uploadMultipartFile(pic);

        return Result.success(sendPic);

    }

    @Override
    public Result<List<AreaVo>> getTree() {
        // 获取顶层节点（省级）
        List<AreaVo> provinces = houseMapper.getProvinces();

        // 递归处理子节点
        for (AreaVo province : provinces) {
            province.setChildren(getChildNodes(province.getId()));
        }

        return Result.success(provinces);
    }

    // 递归获取子节点
    private List<AreaVo> getChildNodes(Integer parentId) {
        // 获取下一级子节点（市级）
        List<AreaVo> cities = houseMapper.getCities(parentId);
        List<AreaVo> chList = new ArrayList<>();

        for (AreaVo city : cities) {
            // 获取下一级子节点（区级）
            city.setChildren(getChildNodes(city.getId()));
            chList.add(city);
        }

        return chList;
    }


}
